@@ -9,50 +9,99 @@ from ticker import NullLocator, LogLocator, AutoLocator, SymmetricalLogLocator,
 from transforms import Transform, IdentityTransform
 
 class ScaleBase(object):
+    """
+    The base class for all scales.
+
+    Scales are separable transformations, working on a single dimension.
+
+    Any subclasses will want to override:
+
+      - :attr:`name`
+      - :meth:`get_transform`
+
+    And optionally:
+      - :meth:`set_default_locators_and_formatters`
+      - :meth:`limit_range_for_scale`
+    """
     def get_transform(self):
         """
-        Return the transform object associated with this scale.
+        Return the :class:`~matplotlib.transforms.Transform` object
+        associated with this scale.
         """
         raise NotImplementedError
 
     def set_default_locators_and_formatters(self, axis):
         """
-        Set the locators and formatters that go with this scale.
+        Set the :class:`~matplotlib.ticker.Locator` and
+        :class:`~matplotlib.ticker.Formatter` objects on the given
+        axis to match this scale.
         """
         raise NotImplementedError
 
     def limit_range_for_scale(self, vmin, vmax, minpos):
         """
-        Returns the range vmin, vmax, limited to the domain supported
-        by this scale.
+        Returns the range *vmin*, *vmax*, possibly limited to the
+        domain supported by this scale.
+
+        *minpos* should be the minimum positive value in the data.
+         This is used by log scales to determine a minimum value.
         """
         return vmin, vmax
 
 class LinearScale(ScaleBase):
+    """
+    The default linear scale.
+    """
+
     name = 'linear'
 
     def __init__(self, axis, **kwargs):
-        """
-        """
         pass
 
     def set_default_locators_and_formatters(self, axis):
+        """
+        Set the locators and formatters to reasonable defaults for
+        linear scaling.
+        """
         axis.set_major_locator(AutoLocator())
         axis.set_major_formatter(ScalarFormatter())
         axis.set_minor_locator(NullLocator())
         axis.set_minor_formatter(NullFormatter())
 
     def get_transform(self):
+        """
+        The transform for linear scaling is just the
+        :class:`~matplotlib.transforms.IdentityTransform`.
+        """
         return IdentityTransform()
 
 
 def _mask_non_positives(a):
+    """
+    Return a Numpy masked array where all non-positive values are
+    masked.  If there are no non-positive values, the original array
+    is returned.
+    """
     mask = a <= 0.0
     if mask.any():
         return ma.MaskedArray(a, mask=mask)
     return a
 
 class LogScale(ScaleBase):
+    """
+    A standard logarithmic scale.  Care is taken so non-positive
+    values are not plotted.
+
+    For computational efficiency (to push as much as possible to Numpy
+    C code in the common cases), this scale provides different
+    transforms depending on the base of the logarithm:
+
+       - base 10 (:class:`Log10Transform`)
+       - base 2 (:class:`Log2Transform`)
+       - base e (:class:`NaturalLogTransform`)
+       - arbitrary base (:class:`LogTransform`)
+    """
+
     name = 'log'
 
     class Log10Transform(Transform):
@@ -91,7 +140,7 @@ class LogScale(ScaleBase):
         def transform(self, a):
             a = _mask_non_positives(a * 2.0)
             if isinstance(a, MaskedArray):
-                return ma.log2(a)
+                return ma.log(a) / np.log(2)
             return np.log2(a)
 
         def inverted(self):
@@ -203,15 +252,26 @@ class LogScale(ScaleBase):
         self.subs = subs
 
     def set_default_locators_and_formatters(self, axis):
+        """
+        Set the locators and formatters to specialized versions for
+        log scaling.
+        """
         axis.set_major_locator(LogLocator(self.base))
         axis.set_major_formatter(LogFormatterMathtext(self.base))
         axis.set_minor_locator(LogLocator(self.base, self.subs))
         axis.set_minor_formatter(NullFormatter())
 
     def get_transform(self):
+        """
+        Return a :class:`~matplotlib.transforms.Transform` instance
+        appropriate for the given logarithm base.
+        """
         return self._transform
 
     def limit_range_for_scale(self, vmin, vmax, minpos):
+        """
+        Limit the domain to positive values.
+        """
         return (vmin <= 0.0 and minpos or vmin,
                 vmax <= 0.0 and minpos or vmax)
 
@@ -221,10 +281,10 @@ class SymmetricalLogScale(ScaleBase):
     The symmetrical logarithmic scale is logarithmic in both the
     positive and negative directions from the origin.
 
-    Since the values close to zero tend toward infinity, there is
-    usually need to have a range around zero that is linear.  The
-    parameter "linthresh" allows the user to specify the size of this
-    range (-linthresh, linthresh).
+    Since the values close to zero tend toward infinity, there is a
+    need to have a range around zero that is linear.  The parameter
+    *linthresh* allows the user to specify the size of this range
+    (-*linthresh*, *linthresh*).
     """
     name = 'symlog'
 
@@ -241,7 +301,8 @@ class SymmetricalLogScale(ScaleBase):
             self._linadjust = (np.log(linthresh) / self._log_base) / linthresh
 
         def transform(self, a):
-            sign = np.sign(np.asarray(a))
+            a = np.asarray(a)
+            sign = np.sign(a)
             masked = ma.masked_inside(a, -self.linthresh, self.linthresh, copy=False)
             log = sign * ma.log(np.abs(masked)) / self._log_base
             if masked.mask.any():
@@ -268,6 +329,7 @@ class SymmetricalLogScale(ScaleBase):
             self._linadjust = linthresh / (np.log(linthresh) / self._log_base)
 
         def transform(self, a):
+            a = np.asarray(a)
             return np.where(a <= self._log_linthresh,
                              np.where(a >= -self._log_linthresh,
                                        a * self._linadjust,
@@ -310,12 +372,19 @@ class SymmetricalLogScale(ScaleBase):
         self.subs = subs
 
     def set_default_locators_and_formatters(self, axis):
+        """
+        Set the locators and formatters to specialized versions for
+        symmetrical log scaling.
+        """
         axis.set_major_locator(SymmetricalLogLocator(self.get_transform()))
         axis.set_major_formatter(LogFormatterMathtext(self.base))
         axis.set_minor_locator(SymmetricalLogLocator(self.get_transform(), self.subs))
         axis.set_minor_formatter(NullFormatter())
 
     def get_transform(self):
+        """
+        Return a :class:`SymmetricalLogTransform` instance.
+        """
         return self._transform
 
 
@@ -325,35 +394,48 @@ _scale_mapping = {
     'log'               : LogScale,
     'symlog'            : SymmetricalLogScale
     }
-def scale_factory(scale, axis, **kwargs):
-    scale = scale.lower()
-    if scale is None:
-        scale = 'linear'
-
-    if not _scale_mapping.has_key(scale):
-        raise ValueError("Unknown scale type '%s'" % scale)
-
-    return _scale_mapping[scale](axis, **kwargs)
-
-def register_scale(scale_class):
-    """
-    Register a new kind of scale.
-    """
-    _scale_mapping[scale_class.name] = scale_class
-
 def get_scale_names():
     names = _scale_mapping.keys()
     names.sort()
     return names
 
+def scale_factory(scale, axis, **kwargs):
+    """
+    Return a scale class by name.
+
+    ACCEPTS: [ %(names)s ]
+    """
+    scale = scale.lower()
+    if scale is None:
+        scale = 'linear'
+
+    if scale not in _scale_mapping:
+        raise ValueError("Unknown scale type '%s'" % scale)
+
+    return _scale_mapping[scale](axis, **kwargs)
+scale_factory.__doc__ = dedent(scale_factory.__doc__) % \
+    {'names': " | ".join(get_scale_names())}
+
+def register_scale(scale_class):
+    """
+    Register a new kind of scale.
+
+    *scale_class* must be a subclass of :class:`ScaleBase`.
+    """
+    _scale_mapping[scale_class.name] = scale_class
+
 def get_scale_docs():
+    """
+    Helper function for generating docstrings related to scales.
+    """
     docs = []
     for name in get_scale_names():
         scale_class = _scale_mapping[name]
         docs.append("    '%s'" % name)
         docs.append("")
         class_docs = dedent(scale_class.__init__.__doc__)
-        class_docs = "".join(["        %s\n" % x for x in class_docs.split("\n")])
+        class_docs = "".join(["        %s\n" %
+                              x for x in class_docs.split("\n")])
         docs.append(class_docs)
         docs.append("")
     return "\n".join(docs)
