@@ -104,7 +104,9 @@ Image::flipud_out(const Py::Tuple& args) {
 
   args.verify_length(0);
   int stride = rbufOut->stride();
+  //std::cout << "flip before: " << rbufOut->stride() << std::endl;
   rbufOut->attach(bufferOut, colsOut, rowsOut, -stride);
+  //std::cout << "flip after: " << rbufOut->stride() << std::endl;
   return Py::Object();
 }
 
@@ -711,7 +713,7 @@ _image_module::from_images(const Py::Tuple& args) {
   size_t numrows = (size_t)Py::Int(args[0]);
   size_t numcols = (size_t)Py::Int(args[1]);
 
-  if (numrows > 1 << 15 || numcols > 1 << 15) {
+  if (numrows >= 32768 || numcols >= 32768) {
     throw Py::RuntimeError("numrows and numcols must both be less than 32768");
   }
 
@@ -744,18 +746,24 @@ _image_module::from_images(const Py::Tuple& args) {
 
 
   rb.clear(agg::rgba(1, 1, 1, 1));
-
   for (size_t imnum=0; imnum< N; imnum++) {
     tup = Py::Tuple(tups[imnum]);
     Image* thisim = static_cast<Image*>(tup[0].ptr());
     ox = Py::Int(tup[1]);
     oy = Py::Int(tup[2]);
-
+    bool isflip = (thisim->rbufOut->stride())<0;
+    //std::cout << "from images " << isflip << "; stride=" << thisim->rbufOut->stride() << std::endl;
     size_t ind=0;
     for (size_t j=0; j<thisim->rowsOut; j++) {
       for (size_t i=0; i<thisim->colsOut; i++) {
 	thisx = i+ox;
-	thisy = j+oy;
+
+	if (isflip)
+	  thisy = thisim->rowsOut - j + oy;
+	else
+	  thisy = j+oy;
+
+
 	if (thisx>=numcols || thisy>=numrows) {
 	  ind +=4;
 	  continue;
@@ -1088,7 +1096,7 @@ _image_module::frombuffer(const Py::Tuple& args) {
   size_t x = Py::Int(args[1]);
   size_t y = Py::Int(args[2]);
 
-  if (x > 1 << 15 || y > 1 << 15) {
+  if (x >= 32768 || y >= 32768) {
     throw Py::ValueError("x and y must both be less than 32768");
   }
 
@@ -1138,145 +1146,18 @@ _image_module::frombuffer(const Py::Tuple& args) {
   return Py::asObject(imo);
 }
 
-
-char __image_module_pcolor__doc__[] =
-"pcolor(x, y, data, rows, cols, bounds)\n"
-"\n"
-"Generate a psudo-color image from data on a non-univorm grid using\n"
-"nearest neighbour interpolation.\n"
-"bounds = (x_min, x_max, y_min, y_max)\n"
-;
-Py::Object
-_image_module::pcolor(const Py::Tuple& args) {
-  _VERBOSE("_image_module::pcolor");
-
-
-  if (args.length() != 6)
-      throw Py::TypeError("Incorrect number of arguments (6 expected)");
-
-  Py::Object xp = args[0];
-  Py::Object yp = args[1];
-  Py::Object dp = args[2];
-  unsigned int rows = Py::Int(args[3]);
-  unsigned int cols = Py::Int(args[4]);
-  Py::Tuple bounds = args[5];
-
-  if (rows > 1 << 15 || cols > 1 << 15) {
-    throw Py::ValueError("rows and cols must both be less than 32768");
-  }
-
-  if (bounds.length() !=4)
-      throw Py::TypeError("Incorrect number of bounds (4 expected)");
-  float x_min = Py::Float(bounds[0]);
-  float x_max = Py::Float(bounds[1]);
-  float y_min = Py::Float(bounds[2]);
-  float y_max = Py::Float(bounds[3]);
-  float width = x_max - x_min;
-  float height = y_max - y_min;
-  float dx = width / ((float) cols);
-  float dy = height / ((float) rows);
-
-  // Check we have something to output to
-  if (rows == 0 || cols ==0)
-      throw Py::ValueError("Cannot scale to zero size");
-
-  // Get numpy arrays
-  PyArrayObject *x = (PyArrayObject *) PyArray_ContiguousFromObject(xp.ptr(), PyArray_FLOAT, 1, 1);
-  if (x == NULL)
-      throw Py::ValueError("x is of incorrect type (wanted 1D float)");
-  PyArrayObject *y = (PyArrayObject *) PyArray_ContiguousFromObject(yp.ptr(), PyArray_FLOAT, 1, 1);
-  if (y == NULL) {
-      Py_XDECREF(x);
-      throw Py::ValueError("y is of incorrect type (wanted 1D float)");
-  }
-  PyArrayObject *d = (PyArrayObject *) PyArray_ContiguousFromObject(dp.ptr(), PyArray_UBYTE, 3, 3);
-  if (d == NULL) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      throw Py::ValueError("data is of incorrect type (wanted 3D UInt8)");
-  }
-  if (d->dimensions[2] != 4) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      Py_XDECREF(d);
-      throw Py::ValueError("data must be in RGBA format");
-  }
-
-  // Check dimensions match
-  int nx = x->dimensions[0];
-  int ny = y->dimensions[0];
-  if (nx != d->dimensions[1] || ny != d->dimensions[0]) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      Py_XDECREF(d);
-      throw Py::ValueError("data and axis dimensions do not match");
-  }
-
-  // Allocate memory for pointer arrays
-  unsigned int * rowstarts = reinterpret_cast<unsigned int*>(PyMem_Malloc(sizeof(unsigned int)*rows));
-  if (rowstarts == NULL) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      Py_XDECREF(d);
-      throw Py::MemoryError("Cannot allocate memory for lookup table");
-  }
-  unsigned int * colstarts = reinterpret_cast<unsigned int*>(PyMem_Malloc(sizeof(unsigned int*)*cols));
-  if (colstarts == NULL) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      Py_XDECREF(d);
-      PyMem_Free(rowstarts);
-      throw Py::MemoryError("Cannot allocate memory for lookup table");
-  }
-
-  // Create output
-  Image* imo = new Image;
-  imo->rowsIn = rows;
-  imo->colsIn = cols;
-  imo->rowsOut = rows;
-  imo->colsOut = cols;
-  size_t NUMBYTES(rows * cols * 4);
-  agg::int8u *buffer = new agg::int8u[NUMBYTES];
-  if (buffer == NULL) {
-      Py_XDECREF(x);
-      Py_XDECREF(y);
-      Py_XDECREF(d);
-      PyMem_Free(rowstarts);
-      PyMem_Free(colstarts);
-      throw Py::MemoryError("Could not allocate memory for image");
-  }
-
-  // Calculate the pointer arrays to map input x to output x
-  unsigned int i, j, j_last;
-  unsigned int * colstart = colstarts;
-  unsigned int * rowstart = rowstarts;
-  float *xs1 = reinterpret_cast<float*>(x->data);
-  float *ys1 = reinterpret_cast<float*>(y->data);
-  float *xs2 = xs1+1;
+// utilities for irregular grids
+void _bin_indices_middle(unsigned int *irows, int nrows, float *ys1, int ny,float dy, float y_min)
+{ int  i,j, j_last;
+   unsigned  int * rowstart = irows;
   float *ys2 = ys1+1;
-  float *xl = xs1 + nx - 1;
-  float *yl = ys1 + ny - 1;
-  float xo = x_min + dx/2.0;
+  float *yl = ys1 + ny ;
   float yo = y_min + dy/2.0;
-  float xm = 0.5*(*xs1 + *xs2);
-  float ym = 0.5*(*ys1 + *ys2);
-  // x/cols
-  j = 0;
-  j_last = j;
-  for (i=0;i<cols;i++,xo+=dx,colstart++) {
-      while(xs2 != xl && xo > xm) {
-          xs1 = xs2;
-          xs2 = xs1+1;
-          xm = 0.5f*(*xs1 + *xs2);
-          j++;
-      }
-      *colstart = j - j_last;
-      j_last = j;
-  }
+  float ym = 0.5f*(*ys1 + *ys2);
   // y/rows
-  j = 0;
-  j_last = j;
-  for (i=0;i<rows;i++,yo+=dy,rowstart++) {
+   j = 0;
+   j_last = j;
+  for (i=0;i<nrows;i++,yo+=dy,rowstart++) {
       while(ys2 != yl && yo > ym) {
           ys1 = ys2;
           ys2 = ys1+1;
@@ -1286,46 +1167,40 @@ _image_module::pcolor(const Py::Tuple& args) {
       *rowstart = j - j_last;
       j_last = j;
   }
+}
 
-
-  // Copy data to output buffer
-  unsigned char *start;
-  unsigned char *inposition;
-  size_t inrowsize(nx*4);
-  size_t rowsize(cols*4);
-  rowstart = rowstarts;
-  agg::int8u * position = buffer;
-  agg::int8u * oldposition = NULL;
-  start = reinterpret_cast<unsigned char*>(d->data);
-  for(i=0;i<rows;i++,rowstart++)
-  {
-      if (i > 0 && *rowstart == 0) {
-          memcpy(position, oldposition, rowsize*sizeof(agg::int8u));
-          oldposition = position;
-          position += rowsize;
-      } else {
-          oldposition = position;
-          start += *rowstart * inrowsize;
-          inposition = start;
-          for(j=0,colstart=colstarts;j<cols;j++,position+=4,colstart++) {
-              inposition += *colstart * 4;
-              memcpy(position, inposition, 4*sizeof(agg::int8u));
-          }
-      }
-  }
-
-  // Attatch output buffer to output buffer
-  imo->rbufOut = new agg::rendering_buffer;
-  imo->bufferOut = buffer;
-  imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
-
-  Py_XDECREF(x);
-  Py_XDECREF(y);
-  Py_XDECREF(d);
-  PyMem_Free(rowstarts);
-  PyMem_Free(colstarts);
-
-  return Py::asObject(imo);
+void _bin_indices_middle_linear(float *arows, unsigned int *irows, int nrows, float *y, int ny,float dy, float y_min)
+{ int i;
+    int ii = 0;
+    int iilast = ny-1;
+    float sc = 1/dy;
+    int iy0 = (int)floor(sc * (y[ii]  - y_min) );
+    int iy1 = (int)floor(sc * (y[ii+1]  - y_min) );
+    float invgap=1.0f/(iy1-iy0);
+    for (i=0; i<nrows && i<=iy0; i++) {
+        irows[i] = 0;
+        arows[i] = 1.0;
+        //std::cerr<<"i="<<i<<"  ii="<<0<<" a="<< arows[i]<< std::endl;
+    }
+    for (; i<nrows; i++) {
+        while (i > iy1 && ii < iilast) {
+            ii++;
+            iy0 = iy1;
+            iy1 = (int)floor(sc * (y[ii+1] - y_min));
+            invgap=1.0f/(iy1-iy0);
+        }
+        if (i >= iy0 && i <= iy1) {
+            irows[i] = ii;
+            arows[i]=(iy1-i)*invgap;
+            //std::cerr<<"i="<<i<<"  ii="<<ii<<" a="<< arows[i]<< std::endl;
+        }
+        else break;
+    }
+    for (; i<nrows; i++) {
+        irows[i] =iilast-1;
+        arows[i] = 0.0;
+        //std::cerr<<"i="<<i<<"  ii="<<iilast-1<<" a="<< arows[i]<< std::endl;
+    }
 }
 
 void _bin_indices(int *irows, int nrows, double *y, int ny,
@@ -1378,6 +1253,298 @@ void _bin_indices(int *irows, int nrows, double *y, int ny,
     }
 }
 
+void _bin_indices_linear(float *arows, int *irows, int nrows, double *y, int ny,
+                                            double sc, double offs)
+{
+    int i;
+    if (sc*(y[ny-1] - y[0]) > 0)
+    {
+        int ii = 0;
+        int iilast = ny-1;
+        int iy0 = (int)floor(sc * (y[ii]  - offs));
+        int iy1 = (int)floor(sc * (y[ii+1]  - offs));
+        float invgap=1.0/(iy1-iy0);
+        for (i=0; i<nrows && i<iy0; i++) {
+            irows[i] = -1;
+        }
+        for (; i<nrows; i++) {
+            while (i > iy1 && ii < iilast) {
+                ii++;
+                iy0 = iy1;
+                iy1 = (int)floor(sc * (y[ii+1] - offs));
+                invgap=1.0/(iy1-iy0);
+            }
+            if (i >= iy0 && i <= iy1) {
+                irows[i] = ii;
+                arows[i]=(iy1-i)*invgap;
+            }
+            else break;
+        }
+        for (; i<nrows; i++) {
+            irows[i] = -1;
+        }
+    }
+    else
+    {
+        int iilast = ny-1;
+        int ii = iilast;
+        int iy0 = (int)floor(sc * (y[ii]  - offs));
+        int iy1 = (int)floor(sc * (y[ii-1]  - offs));
+        float invgap=1.0/(iy1-iy0);
+        for (i=0; i<nrows && i<iy0; i++) {
+            irows[i] = -1;
+        }
+        for (; i<nrows; i++) {
+            while (i > iy1 && ii > 1) {
+                ii--;
+                iy0 = iy1;
+                iy1 = (int)floor(sc * (y[ii-1] - offs));
+                invgap=1.0/(iy1-iy0);
+            }
+            if (i >= iy0 && i <= iy1) {
+                irows[i] = ii-1;
+                arows[i]=(i-iy0)*invgap;
+            }
+            else break;
+        }
+        for (; i<nrows; i++) {
+            irows[i] = -1;
+        }
+    }
+}
+
+
+
+char __image_module_pcolor__doc__[] =
+"pcolor(x, y, data, rows, cols, bounds)\n"
+"\n"
+"Generate a pseudo-color image from data on a non-uniform grid using\n"
+"nearest neighbour or linear interpolation.\n"
+"bounds = (x_min, x_max, y_min, y_max)\n"
+"interpolation = NEAREST or BILINEAR \n"
+;
+
+void _pcolor_cleanup(PyArrayObject* x, PyArrayObject* y,  PyArrayObject *d,
+                                        unsigned int * rowstarts ,unsigned int*colstarts ,
+                                        float *acols , float *arows) {
+    if (x) {
+      Py_XDECREF(x);
+    }
+    if (y) {
+      Py_XDECREF(y);
+    }
+    if(d) {
+      Py_XDECREF(d);
+    }
+    if(rowstarts) {
+      PyMem_Free(rowstarts);
+    }
+    if(colstarts) {
+      PyMem_Free(colstarts);
+    }
+    if(acols) {
+      PyMem_Free(acols);
+    }
+    if(arows) {
+      PyMem_Free(arows);
+    }
+    return;
+}
+
+Py::Object
+_image_module::pcolor(const Py::Tuple& args) {
+  _VERBOSE("_image_module::pcolor");
+
+
+  if (args.length() != 7)
+      throw Py::TypeError("Incorrect number of arguments (7 expected)");
+
+  Py::Object xp = args[0];
+  Py::Object yp = args[1];
+  Py::Object dp = args[2];
+  unsigned int rows = Py::Int(args[3]);
+  unsigned int cols = Py::Int(args[4]);
+  Py::Tuple bounds = args[5];
+  unsigned int interpolation = Py::Int(args[6]);
+
+  if (rows >= 32768 || cols >= 32768) {
+    throw Py::ValueError("rows and cols must both be less than 32768");
+  }
+
+  if (bounds.length() !=4)
+      throw Py::TypeError("Incorrect number of bounds (4 expected)");
+  float x_min = Py::Float(bounds[0]);
+  float x_max = Py::Float(bounds[1]);
+  float y_min = Py::Float(bounds[2]);
+  float y_max = Py::Float(bounds[3]);
+  float width = x_max - x_min;
+  float height = y_max - y_min;
+  float dx = width / ((float) cols);
+  float dy = height / ((float) rows);
+
+  // Check we have something to output to
+  if (rows == 0 || cols ==0)
+      throw Py::ValueError("Cannot scale to zero size");
+
+  PyArrayObject *x = NULL; PyArrayObject *y = NULL; PyArrayObject *d = NULL;
+  unsigned int * rowstarts = NULL; unsigned int*colstarts = NULL;
+  float *acols = NULL; float *arows = NULL;
+
+  // Get numpy arrays
+  x = (PyArrayObject *) PyArray_ContiguousFromObject(xp.ptr(), PyArray_FLOAT, 1, 1);
+  if (x == NULL) {
+      _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::ValueError("x is of incorrect type (wanted 1D float)");
+  }
+  y = (PyArrayObject *) PyArray_ContiguousFromObject(yp.ptr(), PyArray_FLOAT, 1, 1);
+  if (y == NULL) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+     throw Py::ValueError("y is of incorrect type (wanted 1D float)");
+  }
+  d = (PyArrayObject *) PyArray_ContiguousFromObject(dp.ptr(), PyArray_UBYTE, 3, 3);
+  if (d == NULL) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::ValueError("data is of incorrect type (wanted 3D UInt8)");
+  }
+  if (d->dimensions[2] != 4) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::ValueError("data must be in RGBA format");
+  }
+
+  // Check dimensions match
+  int nx = x->dimensions[0];
+  int ny = y->dimensions[0];
+  if (nx != d->dimensions[1] || ny != d->dimensions[0]) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::ValueError("data and axis dimensions do not match");
+  }
+
+  // Allocate memory for pointer arrays
+  rowstarts = reinterpret_cast<unsigned int*>(PyMem_Malloc(sizeof(unsigned int)*rows));
+  if (rowstarts == NULL) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::MemoryError("Cannot allocate memory for lookup table");
+  }
+  colstarts = reinterpret_cast<unsigned int*>(PyMem_Malloc(sizeof(unsigned int)*cols));
+  if (colstarts == NULL) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::MemoryError("Cannot allocate memory for lookup table");
+  }
+
+  // Create output
+  Image* imo = new Image;
+  imo->rowsIn = rows;
+  imo->colsIn = cols;
+  imo->rowsOut = rows;
+  imo->colsOut = cols;
+  size_t NUMBYTES(rows * cols * 4);
+  agg::int8u *buffer = new agg::int8u[NUMBYTES];
+  if (buffer == NULL) {
+     _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+      throw Py::MemoryError("Could not allocate memory for image");
+  }
+
+
+  // Calculate the pointer arrays to map input x to output x
+  unsigned int i, j;
+  unsigned int * colstart = colstarts;
+  unsigned int * rowstart = rowstarts;
+  float *xs1 = reinterpret_cast<float*>(x->data);
+  float *ys1 = reinterpret_cast<float*>(y->data);
+
+
+  // Copy data to output buffer
+  unsigned char *start;
+  unsigned char *inposition;
+  size_t inrowsize(nx*4);
+  size_t rowsize(cols*4);
+  rowstart = rowstarts;
+  agg::int8u * position = buffer;
+  agg::int8u * oldposition = NULL;
+  start = reinterpret_cast<unsigned char*>(d->data);
+    int s0 = d->strides[0];
+    int s1 = d->strides[1];
+
+  if(interpolation == Image::NEAREST) {
+      _bin_indices_middle(colstart, cols, xs1,  nx,dx,x_min);
+      _bin_indices_middle(rowstart, rows, ys1,  ny, dy,y_min);
+      for(i=0;i<rows;i++,rowstart++)
+      {
+          if (i > 0 && *rowstart == 0) {
+              memcpy(position, oldposition, rowsize*sizeof(agg::int8u));
+              oldposition = position;
+              position += rowsize;
+          } else {
+              oldposition = position;
+              start += *rowstart * inrowsize;
+              inposition = start;
+              for(j=0,colstart=colstarts;j<cols;j++,position+=4,colstart++) {
+                  inposition += *colstart * 4;
+                  memcpy(position, inposition, 4*sizeof(agg::int8u));
+              }
+          }
+      }
+  }
+  else if(interpolation == Image::BILINEAR) {
+      arows = reinterpret_cast<float *>(PyMem_Malloc(sizeof(float)*rows));
+      if (arows == NULL ) {
+         _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+          throw Py::MemoryError("Cannot allocate memory for lookup table");
+      }
+      acols = reinterpret_cast<float*>(PyMem_Malloc(sizeof(float)*cols));
+      if (acols == NULL) {
+         _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+          throw Py::MemoryError("Cannot allocate memory for lookup table");
+      }
+
+      _bin_indices_middle_linear(acols, colstart, cols, xs1,  nx,dx,x_min);
+      _bin_indices_middle_linear(arows, rowstart, rows, ys1,  ny, dy,y_min);
+      double a00,a01,a10,a11,alpha,beta;
+
+
+        agg::int8u * start00;
+        agg::int8u * start01;
+        agg::int8u * start10;
+        agg::int8u * start11;
+      // Copy data to output buffer
+      for (i=0; i<rows; i++)
+        {
+            for (j=0; j<cols; j++)
+            {
+                alpha=arows[i];
+                beta=acols[j];
+
+                a00=alpha*beta;
+                a01=alpha*(1.0-beta);
+                a10=(1.0-alpha)*beta;
+                a11=1.0-a00-a01-a10;
+
+                start00=(agg::int8u *)(start + s0*rowstart[i] + s1*colstart[j]);
+                start01=start00+s1;
+                start10=start00+s0;
+                start11=start10+s1;
+                position[0] =(agg::int8u)(start00[0]*a00+start01[0]*a01+start10[0]*a10+start11[0]*a11);
+                position[1] =(agg::int8u)(start00[1]*a00+start01[1]*a01+start10[1]*a10+start11[1]*a11);
+                position[2] =(agg::int8u)(start00[2]*a00+start01[2]*a01+start10[2]*a10+start11[2]*a11);
+                position[3] =(agg::int8u)(start00[3]*a00+start01[3]*a01+start10[3]*a10+start11[3]*a11);
+                position += 4;
+            }
+        }
+
+    }
+
+  // Attach output buffer to output buffer
+  imo->rbufOut = new agg::rendering_buffer;
+  imo->bufferOut = buffer;
+  imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
+
+   _pcolor_cleanup(x,y,d,rowstarts,colstarts,acols,arows);
+
+  return Py::asObject(imo);
+
+}
+
+
 char __image_module_pcolor2__doc__[] =
 "pcolor2(x, y, data, rows, cols, bounds, bg)\n"
 "\n"
@@ -1401,7 +1568,7 @@ _image_module::pcolor2(const Py::Tuple& args) {
     Py::Tuple bounds = args[5];
     Py::Object bgp = args[6];
 
-    if (rows > 1 << 15 || cols > 1 << 15) {
+    if (rows >= 32768 || cols >= 32768) {
       throw Py::ValueError("rows and cols must both be less than 32768");
     }
 
@@ -1477,7 +1644,7 @@ _image_module::pcolor2(const Py::Tuple& args) {
         Py_XDECREF(bg);
         throw Py::MemoryError("Cannot allocate memory for lookup table");
     }
-    int * jcols = reinterpret_cast<int*>(PyMem_Malloc(sizeof(int*)*cols));
+    int * jcols = reinterpret_cast<int*>(PyMem_Malloc(sizeof(int)*cols));
     if (jcols == NULL) {
         Py_XDECREF(x);
         Py_XDECREF(y);
