@@ -148,6 +148,7 @@ GCAgg::GCAgg(const Py::Object &gc, double dpi) :
   _set_dashes(gc);
   _set_clip_rectangle(gc);
   _set_clip_path(gc);
+  _set_snap(gc);
 }
 
 GCAgg::GCAgg(double dpi) :
@@ -254,6 +255,24 @@ GCAgg::_set_clip_path( const Py::Object& gc) {
   }
 }
 
+void
+GCAgg::_set_snap( const Py::Object& gc) {
+  //set the snap setting
+
+  _VERBOSE("GCAgg::_set_snap");
+
+  Py::Object method_obj = gc.getAttr("get_snap");
+  Py::Callable method(method_obj);
+  Py::Object py_snap = method.apply(Py::Tuple());
+  if (py_snap.isNone()) {
+    snap = SNAP_AUTO;
+  } else if (py_snap.isTrue()) {
+    snap = SNAP_TRUE;
+  } else {
+    snap = SNAP_FALSE;
+  }
+}
+
 const size_t
 RendererAgg::PIXELS_PER_INCH(96);
 
@@ -263,63 +282,57 @@ RendererAgg::RendererAgg(unsigned int width, unsigned int height, double dpi,
   height(height),
   dpi(dpi),
   NUMBYTES(width*height*4),
+  pixBuffer(NULL),
+  renderingBuffer(),
   alphaBuffer(NULL),
-  alphaMaskRenderingBuffer(NULL),
-  alphaMask(NULL),
-  pixfmtAlphaMask(NULL),
-  rendererBaseAlphaMask(NULL),
-  rendererAlphaMask(NULL),
-  scanlineAlphaMask(NULL),
+  alphaMaskRenderingBuffer(),
+  alphaMask(alphaMaskRenderingBuffer),
+  pixfmtAlphaMask(alphaMaskRenderingBuffer),
+  rendererBaseAlphaMask(),
+  rendererAlphaMask(),
+  scanlineAlphaMask(),
+  slineP8(),
+  slineBin(),
+  pixFmt(),
+  rendererBase(),
+  rendererAA(),
+  rendererBin(),
+  theRasterizer(),
   debug(debug)
 {
   _VERBOSE("RendererAgg::RendererAgg");
   unsigned stride(width*4);
 
   pixBuffer	  = new agg::int8u[NUMBYTES];
-  renderingBuffer = new agg::rendering_buffer;
-  renderingBuffer->attach(pixBuffer, width, height, stride);
-
-  slineP8  = new scanline_p8;
-  slineBin = new scanline_bin;
-
-  pixFmt       = new pixfmt(*renderingBuffer);
-  rendererBase = new renderer_base(*pixFmt);
-  rendererBase->clear(agg::rgba(1, 1, 1, 0));
-
-  rendererAA	= new renderer_aa(*rendererBase);
-  rendererBin	= new renderer_bin(*rendererBase);
-  theRasterizer = new rasterizer();
-  //theRasterizer->filling_rule(agg::fill_even_odd);
-  //theRasterizer->filling_rule(agg::fill_non_zero);
-
+  renderingBuffer.attach(pixBuffer, width, height, stride);
+  pixFmt.attach(renderingBuffer);
+  rendererBase.attach(pixFmt);
+  rendererBase.clear(agg::rgba(1, 1, 1, 0));
+  rendererAA.attach(rendererBase);
+  rendererBin.attach(rendererBase);
 }
 
 void RendererAgg::create_alpha_buffers() {
   if (!alphaBuffer) {
     unsigned stride(width*4);
     alphaBuffer		   = new agg::int8u[NUMBYTES];
-    alphaMaskRenderingBuffer = new agg::rendering_buffer;
-    alphaMaskRenderingBuffer->attach(alphaBuffer, width, height, stride);
-    alphaMask		   = new alpha_mask_type(*alphaMaskRenderingBuffer);
-
-    pixfmtAlphaMask	   = new agg::pixfmt_gray8(*alphaMaskRenderingBuffer);
-    rendererBaseAlphaMask  = new renderer_base_alpha_mask_type(*pixfmtAlphaMask);
-    rendererAlphaMask	   = new renderer_alpha_mask_type(*rendererBaseAlphaMask);
-    scanlineAlphaMask	   = new agg::scanline_p8();
+    alphaMaskRenderingBuffer.attach(alphaBuffer, width, height, stride);
+    rendererBaseAlphaMask.attach(pixfmtAlphaMask);
+    rendererAlphaMask.attach(rendererBaseAlphaMask);
   }
 }
 
 template<class R>
 void
-RendererAgg::set_clipbox(const Py::Object& cliprect, R rasterizer) {
+RendererAgg::set_clipbox(const Py::Object& cliprect, R& rasterizer) {
   //set the clip rectangle from the gc
 
   _VERBOSE("RendererAgg::set_clipbox");
 
   double l, b, r, t;
   if (py_convert_bbox(cliprect.ptr(), l, b, r, t)) {
-    rasterizer->clip_box(int(mpl_round(l)) + 1, height - int(mpl_round(b)),
-			 int(mpl_round(r)),     height - int(mpl_round(t)));
+    rasterizer.clip_box(int(mpl_round(l)), height - int(mpl_round(b)),
+                        int(mpl_round(r)), height - int(mpl_round(t)));
   }
 
   _VERBOSE("RendererAgg::set_clipbox done");
@@ -341,88 +354,53 @@ RendererAgg::_get_rgba_face(const Py::Object& rgbFace, double alpha) {
   return face;
 }
 
-SnapData
-SafeSnap::snap (const float& x, const float& y) {
-  xsnap = (int)(x + 0.5f);
-  ysnap = (int)(y + 0.5f);
-
-  if ( first || ( (xsnap!=lastxsnap) || (ysnap!=lastysnap) ) ) {
-    lastxsnap = xsnap;
-    lastysnap = ysnap;
-    lastx = x;
-    lasty = y;
-    first = false;
-    return SnapData(true, xsnap, ysnap);
-  }
-
-  // ok both are equal and we need to do an offset
-  if ( (x==lastx) && (y==lasty) ) {
-    // no choice but to return equal coords; set newpoint = false
-    lastxsnap = xsnap;
-    lastysnap = ysnap;
-    lastx = x;
-    lasty = y;
-    return SnapData(false, xsnap, ysnap);
-  }
-
-  // ok the real points are not identical but the rounded ones, so do
-  // a one pixel offset
-  if (x>lastx) xsnap += 1.;
-  else if (x<lastx) xsnap -= 1.;
-
-  if (y>lasty) ysnap += 1.;
-  else if (y<lasty) ysnap -= 1.;
-
-  lastxsnap = xsnap;
-  lastysnap = ysnap;
-  lastx = x;
-  lasty = y;
-  return SnapData(true, xsnap, ysnap);
-}
-
 template<class Path>
-bool should_snap(Path& path, const agg::trans_affine& trans) {
+bool should_snap(GCAgg& gc, Path& path, const agg::trans_affine& trans) {
   // If this contains only straight horizontal or vertical lines, it should be
   // quantized to the nearest pixels
   double x0, y0, x1, y1;
   unsigned code;
 
-  if (path.total_vertices() > 15)
-    return false;
+  switch (gc.snap) {
+  case GCAgg::SNAP_AUTO:
+    if (path.total_vertices() > 15)
+      return false;
 
-  code = path.vertex(&x0, &y0);
-  if (code == agg::path_cmd_stop) {
-    path.rewind(0);
-    return false;
-  }
-  trans.transform(&x0, &y0);
-
-  while ((code = path.vertex(&x1, &y1)) != agg::path_cmd_stop) {
-    trans.transform(&x1, &y1);
-
-    switch (code) {
-    case agg::path_cmd_curve3:
-    case agg::path_cmd_curve4:
+    code = path.vertex(&x0, &y0);
+    if (code == agg::path_cmd_stop) {
       path.rewind(0);
       return false;
-    case agg::path_cmd_line_to:
-      if (!(fabs(x0 - x1) < 1e-4 || fabs(y0 - y1) < 1e-4)) {
-	path.rewind(0);
-	return false;
+    }
+    trans.transform(&x0, &y0);
+
+    while ((code = path.vertex(&x1, &y1)) != agg::path_cmd_stop) {
+      trans.transform(&x1, &y1);
+
+      switch (code) {
+      case agg::path_cmd_curve3:
+      case agg::path_cmd_curve4:
+        path.rewind(0);
+        return false;
+      case agg::path_cmd_line_to:
+        if (!(fabs(x0 - x1) < 1e-4 || fabs(y0 - y1) < 1e-4)) {
+          path.rewind(0);
+          return false;
+        }
       }
+
+      x0 = x1;
+      y0 = y1;
     }
 
-    x0 = x1;
-    y0 = y1;
+    path.rewind(0);
+    gc.isaa = false;
+    return true;
+  case GCAgg::SNAP_FALSE:
+    return false;
+  case GCAgg::SNAP_TRUE:
+    return true;
   }
-
-  path.rewind(0);
-  return true;
-}
-
-template<class Path>
-bool should_simplify(Path& path) {
-  return !path.has_curves() && path.total_vertices() >= 128;
+  return false;
 }
 
 Py::Object
@@ -435,6 +413,7 @@ RendererAgg::copy_from_bbox(const Py::Tuple& args) {
   if (!py_convert_bbox(box_obj.ptr(), l, b, r, t))
     throw Py::TypeError("Invalid bbox provided to copy_from_bbox");
 
+  //  std::cout << l << " " << b << " " << r << " " << t << " " << (height - (int)b) << " " << height - (int)t << std::endl;
   agg::rect_i rect((int)l, height - (int)t, (int)r, height - (int)b);
 
   BufferRegion* reg = NULL;
@@ -448,12 +427,17 @@ RendererAgg::copy_from_bbox(const Py::Tuple& args) {
     throw Py::MemoryError("RendererAgg::copy_from_bbox could not allocate memory for buffer");
   }
 
-  agg::rendering_buffer rbuf;
-  rbuf.attach(reg->data, reg->width, reg->height, reg->stride);
+  try {
+    agg::rendering_buffer rbuf;
+    rbuf.attach(reg->data, reg->width, reg->height, reg->stride);
 
-  pixfmt pf(rbuf);
-  renderer_base rb(pf);
-  rb.copy_from(*renderingBuffer, &rect, -rect.x1, -rect.y1);
+    pixfmt pf(rbuf);
+    renderer_base rb(pf);
+    rb.copy_from(renderingBuffer, &rect, -rect.x1, -rect.y1);
+  } catch (...) {
+    delete reg;
+    throw Py::RuntimeError("An unknown error occurred in copy_from_bbox");
+  }
   return Py::asObject(reg);
 }
 
@@ -464,8 +448,10 @@ RendererAgg::restore_region(const Py::Tuple& args) {
   BufferRegion* region  = static_cast<BufferRegion*>(args[0].ptr());
 
   if (region->data==NULL)
-    return Py::Object();
-  //throw Py::ValueError("Cannot restore_region from NULL data");
+    throw Py::ValueError("Cannot restore_region from NULL data");
+    //return Py::Object();
+
+  //std::cout << "restoring " << region->width << " " << region->height << " " << region->stride << " " << region->rect.x1 << " " << region->rect.y1 << std::endl;
 
   agg::rendering_buffer rbuf;
   rbuf.attach(region->data,
@@ -473,7 +459,7 @@ RendererAgg::restore_region(const Py::Tuple& args) {
 	      region->height,
 	      region->stride);
 
-  rendererBase->copy_from(rbuf, 0, region->rect.x1, region->rect.y1);
+  rendererBase.copy_from(rbuf, 0, region->rect.x1, region->rect.y1);
 
   return Py::Object();
 }
@@ -493,12 +479,12 @@ bool RendererAgg::render_clippath(const Py::Object& clippath, const agg::trans_a
     trans *= agg::trans_affine_translation(0.0, (double)height);
 
     PathIterator clippath_iter(clippath);
-    rendererBaseAlphaMask->clear(agg::gray8(0, 0));
+    rendererBaseAlphaMask.clear(agg::gray8(0, 0));
     transformed_path_t transformed_clippath(clippath_iter, trans);
     agg::conv_curve<transformed_path_t> curved_clippath(transformed_clippath);
-    theRasterizer->add_path(curved_clippath);
-    rendererAlphaMask->color(agg::gray8(255, 255));
-    agg::render_scanlines(*theRasterizer, *scanlineAlphaMask, *rendererAlphaMask);
+    theRasterizer.add_path(curved_clippath);
+    rendererAlphaMask.color(agg::gray8(255, 255));
+    agg::render_scanlines(theRasterizer, scanlineAlphaMask, rendererAlphaMask);
     lastclippath = clippath;
     lastclippath_transform = clippath_trans;
   }
@@ -529,44 +515,47 @@ RendererAgg::draw_markers(const Py::Tuple& args) {
   if (args.size() == 6)
     face_obj = args[5];
 
+  GCAgg gc = GCAgg(gc_obj, dpi);
+
   // Deal with the difference in y-axis direction
   marker_trans *= agg::trans_affine_scaling(1.0, -1.0);
   trans *= agg::trans_affine_scaling(1.0, -1.0);
   trans *= agg::trans_affine_translation(0.0, (double)height);
 
   PathIterator marker_path(marker_path_obj);
-  bool marker_snap = should_snap(marker_path, marker_trans);
+  // The built-in markers look better if snapping is turned on, but
+  // unfortunately, it can cause really small things to disappear.
+  // Disabling for now to revisit at a later date.
+  // const bool marker_snap = true;
+  bool marker_snap = should_snap(gc, marker_path, marker_trans);
   transformed_path_t marker_path_transformed(marker_path, marker_trans);
   simplify_t marker_path_simplified(marker_path_transformed, marker_snap, false, width, height);
   curve_t marker_path_curve(marker_path_simplified);
 
   PathIterator path(path_obj);
   transformed_path_t path_transformed(path, trans);
-  GCAgg gc = GCAgg(gc_obj, dpi);
   path_transformed.rewind(0);
 
   facepair_t face = _get_rgba_face(face_obj, gc.alpha);
 
   //maxim's suggestions for cached scanlines
   agg::scanline_storage_aa8 scanlines;
-  theRasterizer->reset();
-  theRasterizer->reset_clipping();
-  rendererBase->reset_clipping(true);
+  theRasterizer.reset();
+  theRasterizer.reset_clipping();
+  rendererBase.reset_clipping(true);
 
   agg::int8u  staticFillCache[MARKER_CACHE_SIZE];
   agg::int8u  staticStrokeCache[MARKER_CACHE_SIZE];
-  agg::int8u* fillCache = NULL;
-  agg::int8u* strokeCache = NULL;
+  agg::int8u* fillCache = staticFillCache;
+  agg::int8u* strokeCache = staticStrokeCache;
 
   try {
     unsigned fillSize = 0;
     if (face.first) {
-      theRasterizer->add_path(marker_path_curve);
-      agg::render_scanlines(*theRasterizer, *slineP8, scanlines);
+      theRasterizer.add_path(marker_path_curve);
+      agg::render_scanlines(theRasterizer, slineP8, scanlines);
       fillSize = scanlines.byte_size();
-      if (fillSize < MARKER_CACHE_SIZE)
-	fillCache = staticFillCache;
-      else
+      if (fillSize >= MARKER_CACHE_SIZE)
 	fillCache = new agg::int8u[fillSize];
       scanlines.serialize(fillCache);
     }
@@ -575,18 +564,16 @@ RendererAgg::draw_markers(const Py::Tuple& args) {
     stroke.width(gc.linewidth);
     stroke.line_cap(gc.cap);
     stroke.line_join(gc.join);
-    theRasterizer->reset();
-    theRasterizer->add_path(stroke);
-    agg::render_scanlines(*theRasterizer, *slineP8, scanlines);
+    theRasterizer.reset();
+    theRasterizer.add_path(stroke);
+    agg::render_scanlines(theRasterizer, slineP8, scanlines);
     unsigned strokeSize = scanlines.byte_size();
-    if (strokeSize < MARKER_CACHE_SIZE)
-      strokeCache = staticStrokeCache;
-    else
+    if (strokeSize >= MARKER_CACHE_SIZE)
       strokeCache = new agg::int8u[strokeSize];
     scanlines.serialize(strokeCache);
 
-    theRasterizer->reset_clipping();
-    rendererBase->reset_clipping(true);
+    theRasterizer.reset_clipping();
+    rendererBase.reset_clipping(true);
     set_clipbox(gc.cliprect, rendererBase);
     bool has_clippath = render_clippath(gc.clippath, gc.clippath_trans);
 
@@ -597,7 +584,9 @@ RendererAgg::draw_markers(const Py::Tuple& args) {
 
     if (has_clippath) {
       while (path_transformed.vertex(&x, &y) != agg::path_cmd_stop) {
-	pixfmt_amask_type pfa(*pixFmt, *alphaMask);
+        x = mpl_round(x); y = mpl_round(y);
+
+	pixfmt_amask_type pfa(pixFmt, alphaMask);
 	amask_ren_type r(pfa);
 	amask_aa_renderer_type ren(r);
 
@@ -612,15 +601,17 @@ RendererAgg::draw_markers(const Py::Tuple& args) {
       }
     } else {
       while (path_transformed.vertex(&x, &y) != agg::path_cmd_stop) {
+        x = mpl_round(x); y = mpl_round(y);
+
 	if (face.first) {
-	  rendererAA->color(face.second);
+	  rendererAA.color(face.second);
 	  sa.init(fillCache, fillSize, x, y);
-	  agg::render_scanlines(sa, sl, *rendererAA);
+	  agg::render_scanlines(sa, sl, rendererAA);
 	}
 
-	rendererAA->color(gc.color);
+	rendererAA.color(gc.color);
 	sa.init(strokeCache, strokeSize, x, y);
-	agg::render_scanlines(sa, sl, *rendererAA);
+	agg::render_scanlines(sa, sl, rendererAA);
       }
     }
   } catch(...) {
@@ -730,17 +721,16 @@ RendererAgg::draw_text_image(const Py::Tuple& args) {
     y = Py::Int( args[2] );
   }
   catch (Py::TypeError) {
-    //x,y out of range; todo issue warning?
     Py_XDECREF(image_array);
-    return Py::Object();
+    throw Py::TypeError("Invalid input arguments to draw_text_image");
   }
 
   double angle = Py::Float( args[3] );
 
   GCAgg gc = GCAgg(args[4], dpi);
 
-  theRasterizer->reset_clipping();
-  rendererBase->reset_clipping(true);
+  theRasterizer.reset_clipping();
+  rendererBase.reset_clipping(true);
   set_clipbox(gc.cliprect, theRasterizer);
 
   agg::rendering_buffer srcbuf((agg::int8u*)buffer, width, height, width);
@@ -769,10 +759,10 @@ RendererAgg::draw_text_image(const Py::Tuple& args) {
   image_accessor_type ia(pixf_img, 0);
   image_span_gen_type image_span_generator(ia, interpolator, filter);
   span_gen_type output_span_generator(&image_span_generator, gc.color);
-  renderer_type ri(*rendererBase, sa, output_span_generator);
+  renderer_type ri(rendererBase, sa, output_span_generator);
 
-  theRasterizer->add_path(rect2);
-  agg::render_scanlines(*theRasterizer, *slineP8, ri);
+  theRasterizer.add_path(rect2);
+  agg::render_scanlines(theRasterizer, slineP8, ri);
 
   Py_XDECREF(image_array);
 
@@ -780,7 +770,6 @@ RendererAgg::draw_text_image(const Py::Tuple& args) {
 }
 
 
-// MGDTODO: Support clip paths
 Py::Object
 RendererAgg::draw_image(const Py::Tuple& args) {
   _VERBOSE("RendererAgg::draw_image");
@@ -793,20 +782,57 @@ RendererAgg::draw_image(const Py::Tuple& args) {
   Py::Object box_obj = args[3];
   Py::Object clippath;
   agg::trans_affine clippath_trans;
+  bool has_clippath = false;
+
+  theRasterizer.reset_clipping();
+  rendererBase.reset_clipping(true);
   if (args.size() == 6) {
     clippath = args[4];
     clippath_trans = py_to_agg_transformation_matrix(args[5], false);
+    has_clippath = render_clippath(clippath, clippath_trans);
   }
 
-  theRasterizer->reset_clipping();
-  rendererBase->reset_clipping(true);
-  set_clipbox(box_obj, rendererBase);
-
   Py::Tuple empty;
-  pixfmt pixf(*(image->rbufOut));
   image->flipud_out(empty);
+  pixfmt pixf(*(image->rbufOut));
 
-  rendererBase->blend_from(pixf, 0, (int)x, (int)(height-(y+image->rowsOut)));
+  if (has_clippath) {
+    agg::trans_affine mtx;
+    mtx *= agg::trans_affine_translation((int)x, (int)(height-(y+image->rowsOut)));
+
+    agg::path_storage rect;
+    rect.move_to(0, 0);
+    rect.line_to(image->colsOut, 0);
+    rect.line_to(image->colsOut, image->rowsOut);
+    rect.line_to(0, image->rowsOut);
+    rect.line_to(0, 0);
+    agg::conv_transform<agg::path_storage> rect2(rect, mtx);
+
+    agg::trans_affine inv_mtx(mtx);
+    inv_mtx.invert();
+
+    typedef agg::span_allocator<agg::rgba8> color_span_alloc_type;
+    typedef agg::pixfmt_amask_adaptor<pixfmt, alpha_mask_type> pixfmt_amask_type;
+    typedef agg::renderer_base<pixfmt_amask_type> amask_ren_type;
+    typedef agg::image_accessor_clip<agg::pixfmt_rgba32> image_accessor_type;
+    typedef agg::span_interpolator_linear<> interpolator_type;
+    typedef agg::span_image_filter_rgba_nn<image_accessor_type, interpolator_type> image_span_gen_type;
+    typedef agg::renderer_scanline_aa<amask_ren_type, color_span_alloc_type, image_span_gen_type> renderer_type;
+
+    color_span_alloc_type sa;
+    image_accessor_type ia(pixf, agg::rgba8(0, 0, 0, 0));
+    interpolator_type interpolator(inv_mtx);
+    image_span_gen_type image_span_generator(ia, interpolator);
+    pixfmt_amask_type pfa(pixFmt, alphaMask);
+    amask_ren_type r(pfa);
+    renderer_type ri(r, sa, image_span_generator);
+
+    theRasterizer.add_path(rect2);
+    agg::render_scanlines(theRasterizer, slineP8, ri);
+  } else {
+    set_clipbox(box_obj, rendererBase);
+    rendererBase.blend_from(pixf, 0, (int)x, (int)(height-(y+image->rowsOut)));
+  }
 
   image->flipud_out(empty);
 
@@ -826,29 +852,29 @@ void RendererAgg::_draw_path(path_t& path, bool has_clippath,
 
   // Render face
   if (face.first) {
-    theRasterizer->add_path(path);
+    theRasterizer.add_path(path);
 
     if (gc.isaa) {
       if (has_clippath) {
-	pixfmt_amask_type pfa(*pixFmt, *alphaMask);
+	pixfmt_amask_type pfa(pixFmt, alphaMask);
 	amask_ren_type r(pfa);
 	amask_aa_renderer_type ren(r);
 	ren.color(face.second);
-	agg::render_scanlines(*theRasterizer, *slineP8, ren);
+	agg::render_scanlines(theRasterizer, slineP8, ren);
       } else {
-	rendererAA->color(face.second);
-	agg::render_scanlines(*theRasterizer, *slineP8, *rendererAA);
+	rendererAA.color(face.second);
+	agg::render_scanlines(theRasterizer, slineP8, rendererAA);
       }
     } else {
       if (has_clippath) {
-	pixfmt_amask_type pfa(*pixFmt, *alphaMask);
+	pixfmt_amask_type pfa(pixFmt, alphaMask);
 	amask_ren_type r(pfa);
 	amask_bin_renderer_type ren(r);
 	ren.color(face.second);
-	agg::render_scanlines(*theRasterizer, *slineP8, ren);
+	agg::render_scanlines(theRasterizer, slineP8, ren);
       } else {
-	rendererBin->color(face.second);
-	agg::render_scanlines(*theRasterizer, *slineP8, *rendererBin);
+	rendererBin.color(face.second);
+	agg::render_scanlines(theRasterizer, slineP8, rendererBin);
       }
     }
   }
@@ -864,7 +890,7 @@ void RendererAgg::_draw_path(path_t& path, bool has_clippath,
       stroke.width(linewidth);
       stroke.line_cap(gc.cap);
       stroke.line_join(gc.join);
-      theRasterizer->add_path(stroke);
+      theRasterizer.add_path(stroke);
     } else {
       dash_t dash(path);
       for (GCAgg::dash_t::const_iterator i = gc.dashes.begin();
@@ -881,30 +907,30 @@ void RendererAgg::_draw_path(path_t& path, bool has_clippath,
       stroke.line_cap(gc.cap);
       stroke.line_join(gc.join);
       stroke.width(linewidth);
-      theRasterizer->add_path(stroke);
+      theRasterizer.add_path(stroke);
     }
 
     if (gc.isaa) {
       if (has_clippath) {
-	pixfmt_amask_type pfa(*pixFmt, *alphaMask);
+	pixfmt_amask_type pfa(pixFmt, alphaMask);
 	amask_ren_type r(pfa);
 	amask_aa_renderer_type ren(r);
 	ren.color(gc.color);
-	agg::render_scanlines(*theRasterizer, *slineP8, ren);
+	agg::render_scanlines(theRasterizer, slineP8, ren);
       } else {
-	rendererAA->color(gc.color);
-	agg::render_scanlines(*theRasterizer, *slineP8, *rendererAA);
+	rendererAA.color(gc.color);
+	agg::render_scanlines(theRasterizer, slineP8, rendererAA);
       }
     } else {
       if (has_clippath) {
-	pixfmt_amask_type pfa(*pixFmt, *alphaMask);
+	pixfmt_amask_type pfa(pixFmt, alphaMask);
 	amask_ren_type r(pfa);
 	amask_bin_renderer_type ren(r);
 	ren.color(gc.color);
-	agg::render_scanlines(*theRasterizer, *slineP8, ren);
+	agg::render_scanlines(theRasterizer, slineP8, ren);
       } else {
-	rendererBin->color(gc.color);
-	agg::render_scanlines(*theRasterizer, *slineBin, *rendererBin);
+	rendererBin.color(gc.color);
+	agg::render_scanlines(theRasterizer, slineBin, rendererBin);
       }
     }
   }
@@ -930,23 +956,25 @@ RendererAgg::draw_path(const Py::Tuple& args) {
   GCAgg gc = GCAgg(gc_obj, dpi);
   facepair_t face = _get_rgba_face(face_obj, gc.alpha);
 
-  theRasterizer->reset_clipping();
-  rendererBase->reset_clipping(true);
+  theRasterizer.reset_clipping();
+  rendererBase.reset_clipping(true);
   set_clipbox(gc.cliprect, theRasterizer);
   bool has_clippath = render_clippath(gc.clippath, gc.clippath_trans);
 
   trans *= agg::trans_affine_scaling(1.0, -1.0);
   trans *= agg::trans_affine_translation(0.0, (double)height);
-  bool snap = should_snap(path, trans);
-  bool simplify = should_simplify(path) && !face.first;
+  bool snap = should_snap(gc, path, trans);
+  bool simplify = path.should_simplify() && !face.first;
 
   transformed_path_t tpath(path, trans);
   simplify_t simplified(tpath, snap, simplify, width, height);
   curve_t curve(simplified);
-  if (snap)
-    gc.isaa = false;
 
-  _draw_path(curve, has_clippath, face, gc);
+  try {
+    _draw_path(curve, has_clippath, face, gc);
+  } catch (const char* e) {
+    throw Py::RuntimeError(e);
+  }
 
   return Py::Object();
 }
@@ -1039,8 +1067,8 @@ RendererAgg::_draw_path_collection_generic
     }
 
     // Handle any clipping globally
-    theRasterizer->reset_clipping();
-    rendererBase->reset_clipping(true);
+    theRasterizer.reset_clipping();
+    rendererBase.reset_clipping(true);
     set_clipbox(cliprect, theRasterizer);
     bool has_clippath = render_clippath(clippath, clippath_trans);
 
@@ -1097,11 +1125,8 @@ RendererAgg::_draw_path_collection_generic
       }
 
       if (check_snap) {
-	snap = should_snap(path, trans);
-	if (snap)
-	  gc.isaa = false;
-	else
-	  gc.isaa = bool(Py::Int(antialiaseds[i % Naa]));
+	snap = should_snap(gc, path, trans);
+        gc.isaa = bool(Py::Int(antialiaseds[i % Naa]));
 
 	transformed_path_t tpath(path, trans);
 	simplify_t simplified(tpath, snap, false, width, height);
@@ -1161,7 +1186,7 @@ public:
 Py::Object
 RendererAgg::draw_path_collection(const Py::Tuple& args) {
   _VERBOSE("RendererAgg::draw_path_collection");
-  args.verify_length(13);
+  args.verify_length(14);
 
   //segments, trans, clipbox, colors, linewidths, antialiaseds
   agg::trans_affine	  master_transform = py_to_agg_transformation_matrix(args[0]);
@@ -1177,23 +1202,28 @@ RendererAgg::draw_path_collection(const Py::Tuple& args) {
   Py::SeqBase<Py::Float>  linewidths	   = args[10];
   Py::SeqBase<Py::Object> linestyles_obj   = args[11];
   Py::SeqBase<Py::Int>    antialiaseds	   = args[12];
-
+  // We don't actually care about urls for Agg, so just ignore it.
+  // Py::SeqBase<Py::Object> urls             = args[13];
   PathListGenerator path_generator(paths);
 
-  _draw_path_collection_generic<PathListGenerator, 1, 1>
-    (master_transform,
-     cliprect,
-     clippath,
-     clippath_trans,
-     path_generator,
-     transforms_obj,
-     offsets_obj,
-     offset_trans,
-     facecolors_obj,
-     edgecolors_obj,
-     linewidths,
-     linestyles_obj,
-     antialiaseds);
+  try {
+    _draw_path_collection_generic<PathListGenerator, 0, 1>
+      (master_transform,
+       cliprect,
+       clippath,
+       clippath_trans,
+       path_generator,
+       transforms_obj,
+       offsets_obj,
+       offset_trans,
+       facecolors_obj,
+       edgecolors_obj,
+       linewidths,
+       linestyles_obj,
+       antialiaseds);
+  } catch (const char *e) {
+    throw Py::RuntimeError(e);
+  }
 
   return Py::Object();
 }
@@ -1214,8 +1244,8 @@ class QuadMeshGenerator {
 
   private:
     inline unsigned vertex(unsigned idx, double* x, double* y) {
-      size_t m = (idx   & 0x2) ? (m_m + 1) : m_m;
-      size_t n = (idx+1 & 0x2) ? (m_n + 1) : m_n;
+      size_t m = m_m + ((idx     & 0x2) >> 1);
+      size_t n = m_n + (((idx+1) & 0x2) >> 1);
       double* pair = (double*)PyArray_GETPTR2(m_coordinates, n, m);
       *x = *pair++;
       *y = *pair;
@@ -1236,6 +1266,10 @@ class QuadMeshGenerator {
     inline unsigned total_vertices() {
       return 5;
     }
+
+    inline bool should_simplify() {
+      return false;
+    }
   };
 
 public:
@@ -1243,7 +1277,7 @@ public:
 
   inline QuadMeshGenerator(size_t meshWidth, size_t meshHeight, PyObject* coordinates) :
     m_meshWidth(meshWidth), m_meshHeight(meshHeight), m_coordinates(NULL) {
-    PyArrayObject* coordinates_array = (PyArrayObject*)PyArray_FromObject(coordinates, PyArray_DOUBLE, 3, 3);
+    PyArrayObject* coordinates_array = (PyArrayObject*)PyArray_ContiguousFromObject(coordinates, PyArray_DOUBLE, 3, 3);
     if (!coordinates_array) {
       throw Py::ValueError("Invalid coordinates array.");
     }
@@ -1310,23 +1344,35 @@ RendererAgg::draw_quad_mesh(const Py::Tuple& args) {
     }
   }
 
-  _draw_path_collection_generic<QuadMeshGenerator, 0, 0>
-    (master_transform,
-     cliprect,
-     clippath,
-     clippath_trans,
-     path_generator,
-     transforms_obj,
-     offsets_obj,
-     offset_trans,
-     facecolors_obj,
-     edgecolors_obj,
-     linewidths,
-     linestyles_obj,
-     antialiaseds);
+  try {
+    try {
+      _draw_path_collection_generic<QuadMeshGenerator, 0, 0>
+        (master_transform,
+         cliprect,
+         clippath,
+         clippath_trans,
+         path_generator,
+         transforms_obj,
+         offsets_obj,
+         offset_trans,
+         facecolors_obj,
+         edgecolors_obj,
+         linewidths,
+         linestyles_obj,
+         antialiaseds);
+    } catch (const char* e) {
+      throw Py::RuntimeError(e);
+    }
+  } catch (...) {
+    if (free_edgecolors) {
+      Py_XDECREF(edgecolors_obj.ptr());
+    }
+    throw;
+  }
 
-  if (free_edgecolors)
+  if (free_edgecolors) {
     Py_XDECREF(edgecolors_obj.ptr());
+  }
 
   return Py::Object();
 }
@@ -1379,21 +1425,28 @@ RendererAgg::tostring_rgb(const Py::Tuple& args) {
   args.verify_length(0);
   int row_len = width*3;
   unsigned char* buf_tmp = new unsigned char[row_len * height];
-  if (buf_tmp ==NULL) {
+  if (buf_tmp == NULL) {
     //todo: also handle allocation throw
     throw Py::MemoryError("RendererAgg::tostring_rgb could not allocate memory");
   }
-  agg::rendering_buffer renderingBufferTmp;
-  renderingBufferTmp.attach(buf_tmp,
-			    width,
-			    height,
-			    row_len);
 
-  agg::color_conv(&renderingBufferTmp, renderingBuffer, agg::color_conv_rgba32_to_rgb24());
+  try {
+    agg::rendering_buffer renderingBufferTmp;
+    renderingBufferTmp.attach(buf_tmp,
+                              width,
+                              height,
+                              row_len);
 
+    agg::color_conv(&renderingBufferTmp, &renderingBuffer, agg::color_conv_rgba32_to_rgb24());
+
+  } catch (...) {
+    delete [] buf_tmp;
+    throw Py::RuntimeError("Unknown exception occurred in tostring_rgb");
+  }
 
   //todo: how to do this with native CXX
   PyObject* o = Py_BuildValue("s#", buf_tmp, row_len * height);
+
   delete [] buf_tmp;
   return Py::asObject(o);
 }
@@ -1412,11 +1465,15 @@ RendererAgg::tostring_argb(const Py::Tuple& args) {
     //todo: also handle allocation throw
     throw Py::MemoryError("RendererAgg::tostring_argb could not allocate memory");
   }
-  agg::rendering_buffer renderingBufferTmp;
-  renderingBufferTmp.attach(buf_tmp, width, height, row_len);
-
-  agg::color_conv(&renderingBufferTmp, renderingBuffer, agg::color_conv_rgba32_to_argb32());
 
+  try {
+    agg::rendering_buffer renderingBufferTmp;
+    renderingBufferTmp.attach(buf_tmp, width, height, row_len);
+    agg::color_conv(&renderingBufferTmp, &renderingBuffer, agg::color_conv_rgba32_to_argb32());
+  } catch (...) {
+    delete [] buf_tmp;
+    throw Py::RuntimeError("Unknown exception occurred in tostring_argb");
+  }
 
   //todo: how to do this with native CXX
   PyObject* o = Py_BuildValue("s#", buf_tmp, row_len * height);
@@ -1437,14 +1494,19 @@ RendererAgg::tostring_bgra(const Py::Tuple& args) {
     //todo: also handle allocation throw
     throw Py::MemoryError("RendererAgg::tostring_bgra could not allocate memory");
   }
-  agg::rendering_buffer renderingBufferTmp;
-  renderingBufferTmp.attach(buf_tmp,
-			    width,
-			    height,
-			    row_len);
 
-  agg::color_conv(&renderingBufferTmp, renderingBuffer, agg::color_conv_rgba32_to_bgra32());
+  try {
+    agg::rendering_buffer renderingBufferTmp;
+    renderingBufferTmp.attach(buf_tmp,
+                              width,
+                              height,
+                              row_len);
 
+    agg::color_conv(&renderingBufferTmp, &renderingBuffer, agg::color_conv_rgba32_to_bgra32());
+  } catch (...) {
+    delete [] buf_tmp;
+    throw Py::RuntimeError("Unknown exception occurred in tostring_bgra");
+  }
 
   //todo: how to do this with native CXX
   PyObject* o = Py_BuildValue("s#",
@@ -1506,12 +1568,17 @@ RendererAgg::tostring_rgba_minimized(const Py::Tuple& args) {
     int newsize	= newwidth * newheight * 4;
 
     unsigned char* buf = new unsigned char[newsize];
+    if (buf == NULL) {
+      throw Py::MemoryError("RendererAgg::tostring_minimized could not allocate memory");
+    }
+
     unsigned int*  dst = (unsigned int*)buf;
     unsigned int*  src = (unsigned int*)pixBuffer;
     for (int y = ymin; y < ymax; ++y)
       for (int x = xmin; x < xmax; ++x, ++dst)
 	*dst = src[y * width + x];
 
+    // The Py::String will take over the buffer
     data = Py::String((const char *)buf, (int)newsize);
   }
 
@@ -1535,7 +1602,7 @@ RendererAgg::clear(const Py::Tuple& args) {
   _VERBOSE("RendererAgg::clear");
 
   args.verify_length(0);
-  rendererBase->clear(agg::rgba(1, 1, 1, 0));
+  rendererBase.clear(agg::rgba(1, 1, 1, 0));
 
   return Py::Object();
 }
@@ -1576,25 +1643,8 @@ RendererAgg::~RendererAgg() {
 
   _VERBOSE("RendererAgg::~RendererAgg");
 
-
-  delete slineP8;
-  delete slineBin;
-  delete theRasterizer;
-  delete rendererAA;
-  delete rendererBin;
-  delete rendererBase;
-  delete pixFmt;
-  delete renderingBuffer;
-
-  delete alphaMask;
-  delete alphaMaskRenderingBuffer;
   delete [] alphaBuffer;
   delete [] pixBuffer;
-  delete pixfmtAlphaMask;
-  delete rendererBaseAlphaMask;
-  delete rendererAlphaMask;
-  delete scanlineAlphaMask;
-
 }
 
 /* ------------ module methods ------------- */
@@ -1701,5 +1751,4 @@ DL_EXPORT(void)
 
   static _backend_agg_module* _backend_agg = NULL;
   _backend_agg = new _backend_agg_module;
-
 }
